#!
# -*- coding: utf-8 -*-
"""
Macro pour faciliter l'intégration des données d'une feuille de calcul
vers un wiki MoinMoin.

Copyright : Agence universitaire de la Francophonie
Licence : GNU General Public Licence, version 2
Auteur : Jean Christophe André
Date de création : septembre 2009
"""
import uno
import unohelper
from com.sun.star.task import XJobExecutor
from com.sun.star.beans import PropertyValue
from com.sun.star.table.CellContentType import EMPTY, VALUE, TEXT, FORMULA
from com.sun.star.table.CellHoriJustify import STANDARD, LEFT, CENTER, RIGHT
from com.sun.star.table.CellVertJustify import STANDARD, TOP, CENTER as MIDDLE, BOTTOM
from com.sun.star.awt.FontWeight import DONTKNOW, NORMAL, BOLD

_settings = {
  'MoinMoinOldStyle': True,
  'MoinMoinVertOnMulti': True,
}

def getRangeSize(sheet, cursor):
    r = cursor.getRangeAddress()
    realRange = sheet.getCellRangeByPosition(r.StartColumn, r.StartRow,
                                             r.EndColumn, r.EndRow)
    return (realRange.Columns.Count, realRange.Rows.Count)

def getCellRealPosition(cursor, column, row):
    r = cursor.getRangeAddress()
    return (r.StartColumn + column, r.StartRow + row)

def cell2moin(cell):
    global _settings
    cell_string = cell.getString()
    if not cell_string: return ''
    cell_string = cell_string.replace('\n','<<BR>>')
    if _settings['MoinMoinOldStyle']:
        if hasattr(cell, 'CharWeight') and cell.CharWeight == BOLD:
            cell_string = "'''%s'''" % cell_string
    return cell_string

def calc2moin(ctx):
    global _settings
    data = list()

    # récupération du document en cours, vérification du type Spreadsheet
    smgr = ctx.ServiceManager
    desktop = smgr.createInstanceWithContext(
        "com.sun.star.frame.Desktop", ctx)
    curdoc = desktop.getCurrentComponent()
    if not curdoc.supportsService("com.sun.star.sheet.SpreadsheetDocument"):
        raise RuntimeError(u"Ce n'est pas un document de type Spreadsheet (Calc).")
    controller = curdoc.getCurrentController()
    sheet = controller.getActiveSheet()
    #open('/tmp/calc2moin.log','a').write('sheet=[%s]\n' % ', '.join(dir(sheet)))

    # localisation de la la zone déjà utilisée
    cursor = sheet.createCursor()
    cursor.gotoEndOfUsedArea(False)
    cursor.gotoStartOfUsedArea(True)
    columns, rows = getRangeSize(sheet, cursor)
    column_rowspan = {}
    for row in range(rows):
        row_data = list()
        row_has_multiline_cell = False
        column = 0
        while column < columns:
            # saut des colonnes cachées par un rowspan précédent
            if column_rowspan.get(column, 0) > 0:
                column_rowspan[column] -= 1
                column += 1
                continue
            # récupération des infos sur la cellule courante
            cell = cursor.getCellByPosition(column, row)
            cell_type = cell.getType()
            cell_string = cell.getString()
            if not row_has_multiline_cell:
                row_has_multiline_cell = '\n' in cell_string
            # calcul de l'étendue de la cellule courante
            if cell.getIsMerged():
                c, r = getCellRealPosition(cursor, column, row)
                cellRange = sheet.getCellRangeByPosition(c, r, c, r)
                cursor2 = sheet.createCursorByRange(cellRange)
                cursor2.collapseToMergedArea()
                cell_colspan, cell_rowspan = getRangeSize(sheet, cursor2)
                for n in range(0, cell_colspan):
                    column_rowspan[column + n] = cell_rowspan - 1
                #open('/tmp/calc2moin.log','a').write('Merged(%d,%d)=[%d,%d]\n' % (column,row,cell_colspan,cell_rowspan))
            else:
                cell_colspan, cell_rowspan = 1, 1
            # traitement des attributs et styles de la cellule
            cell_attributes = list()
            cell_styles = list()
            if cell_string and hasattr(cell, 'HoriJustify'):
                horiJustify = cell.getPropertyValue('HoriJustify')
                if horiJustify == LEFT:
                    if cell_colspan > 1 or cell_rowspan > 1:
                        text_align = 'left'
                        text_align_old = '('
                    else:
                        text_align = '' # left par défaut si span == 1
                        text_align_old = ''
                elif horiJustify == CENTER:
                    if cell_colspan == 1 and cell_rowspan == 1:
                        text_align = 'center'
                        text_align_old = ':'
                    else:
                        text_align = '' # center par défaut si span > 1
                        text_align_old = ''
                elif horiJustify == RIGHT:
                    text_align = 'right'
                    text_align_old = ')'
                else:
                    if cell_type == TEXT:
                        if cell_colspan > 1 or cell_rowspan > 1:
                            text_align = 'left'
                            text_align_old = '('
                        else:
                            text_align = '' # left par défaut si span == 1
                            text_align_old = ''
                    elif cell_type != EMPTY:
                        text_align = 'right'
                        text_align_old = ')'
                if text_align:
                    if _settings['MoinMoinOldStyle']:
                        cell_attributes.append(text_align_old)
                    else:
                        cell_styles.append('text-align: ' + text_align)
            if cell_string and hasattr(cell, 'VertJustify') and \
              (row_has_multiline_cell or not _settings['MoinMoinVertOnMulti']):
                vertJustify = cell.getPropertyValue('VertJustify')
                if vertJustify == 1: #TOP(1)
                    vertical_align = 'top'
                    vertical_align_old = '^'
                elif vertJustify == 2: #MIDDLE(2)
                    vertical_align = '' # middle par défaut
                    vertical_align_old = ''
                else: #STANDARD(0), BOTTOM(3)
                    vertical_align = 'bottom'
                    vertical_align_old = 'v'
                if vertical_align:
                    if _settings['MoinMoinOldStyle']:
                        cell_attributes.append(vertical_align_old)
                    else:
                        cell_styles.append('vertical-align: ' + vertical_align)
            if cell_string and hasattr(cell, 'CharWeight'):
                charWeight = cell.getPropertyValue('CharWeight')
                if charWeight == BOLD and not _settings['MoinMoinOldStyle']:
                    cell_styles.append('font-weight: bold')
            if cell_rowspan > 1:
                cell_attributes.append('|%d' % cell_rowspan)
            if cell_colspan > 1:
                cell_attributes.append('-%d' % cell_colspan)
            if hasattr(cell, 'CellBackColor'):
                color = cell.getPropertyValue('CellBackColor')
                if color >= 0:
                    if _settings['MoinMoinOldStyle']:
                        cell_attributes.append('#%06x' % color)
                    else:
                        cell_styles.append('background: #%06x' % color)
            if hasattr(cell, 'CharColor'):
                color = cell.getPropertyValue('CharColor')
                if color >= 0:
                    cell_styles.append('color: #%06x' % color)
            # compilation des styles de la cellule
            if cell_styles:
                cell_attributes.insert(0, 'style="%s;"' % '; '.join(cell_styles))
            # ajout de la définition de la cellule à la ligne courante
            cell_data = list()
            if cell_attributes:
                cell_data.append('<' + ''.join(cell_attributes) + '>')
            if cell_string:
                cell_data.append(cell2moin(cell))
            else:
                cell_data.append(' ')
            row_data.append(''.join(cell_data))
            column += cell_colspan
        # display row's code
        data.append('||' + '||'.join(row_data) + '||')

    #
    # création d'un document Writer pour écrire le code MoinMoin
    #
    # ouverture d'un document Writer caché
    hidden = PropertyValue()
    hidden.Name = "Hidden"
    hidden.Value = True
    doc = desktop.loadComponentFromURL(
        "private:factory/swriter", "_blank", 0, (hidden, ) )
    text = doc.Text
    textcursor = text.createTextCursor()
    text.insertString(textcursor, '\n'.join(data) + '\n', 0)
    # on copie ça dans le presse papier
    dispatcher = smgr.createInstanceWithContext(
        "com.sun.star.frame.DispatchHelper", ctx)
    frame = doc.getCurrentController().getFrame()
    dispatcher.executeDispatch(frame, ".uno:SelectAll", "", 0, ())
    dispatcher.executeDispatch(frame, ".uno:Copy", "", 0, ())
    doc.close(True)

##############################################################################

def copier(event=False):
    u"""Copier une feuille de calcul vers un wiki MoinMoin."""
    ctx = uno.getComponentContext()
    calc2moin(ctx)
    return None

# lists the scripts, that shall be visible inside OOo. Can be omited, if
# all functions shall be visible, however here getNewString shall be surpressed
g_exportedScripts = (copier, )

##############################################################################

class CopierJob(unohelper.Base, XJobExecutor):
    def __init__(self, context):
        self._context = context
        self._readConfig()

    def _readConfig(self):
        cfgprov = self._context.ServiceManager.createInstanceWithContext(
            "com.sun.star.configuration.ConfigurationProvider", self._context)
        node = PropertyValue()
        node.Name = "nodepath"
        node.Value = "/org.libreoffice.moinmoin/General"
        ConfigReader = cfgprov.createInstanceWithArguments(
            "com.sun.star.configuration.ConfigurationAccess", (node,))
        cfgnames = ("MoinMoinOldStyle", "MoinMoinVertOnMulti", )
        cfgvalues = ConfigReader.getPropertyValues(cfgnames)
        if not cfgvalues:
            raise RuntimeError("Unable to read the configuration.")
        global _settings
        _settings = dict(zip(cfgnames, cfgvalues))

    def trigger(self, args):
        calc2moin(self._context)

g_ImplementationHelper = unohelper.ImplementationHelper()
g_ImplementationHelper.addImplementation( \
    CopierJob, "org.libreoffice.moinmoin.calc2moin.Copier", \
    ("com.sun.star.task.Job",),)
